from django.test.simple import DjangoTestSuiteRunner
import pdb

try:
    # Django 1.3+
    from django.utils import unittest
    TextTestResult = unittest.TextTestResult
except ImportError:
    # Django 1.2
    import unittest
    TextTestResult = unittest._TextTestResult
    from django.test.simple import DjangoTestRunner
else:
    # Django 1.3+
    from django.utils.unittest import TextTestRunner as DjangoTestRunner

from django_pdb.utils import has_ipdb


class ExceptionTestResultMixin(object):
    """
    A mixin class that can be added to any test result class.
    Drops into pdb on test errors/failures.
    """
    pdb_type = 'pdb'

    def get_pdb(self):
        if self.pdb_type == 'ipdb' and has_ipdb():
            import ipdb
            return ipdb
        return pdb

    def addError(self, test, err):
        super(ExceptionTestResultMixin, self).addError(test, err)
        exctype, value, tb = err

        self.stream.writeln()
        self.stream.writeln(self.separator1)
        self.stream.writeln(">>> %s" % (self.getDescription(test)))
        self.stream.writeln(self.separator2)
        self.stream.writeln(self._exc_info_to_string(err, test).rstrip())
        self.stream.writeln(self.separator1)
        self.stream.writeln()

        # Skip test runner traceback levels
        #while tb and self._is_relevant_tb_level(tb):
        #    tb = tb.tb_next

        self.get_pdb().post_mortem(tb)

    def addFailure(self, test, err):
        super(ExceptionTestResultMixin, self).addFailure(test, err)
        exctype, value, tb = err

        self.stream.writeln()
        self.stream.writeln(self.separator1)
        self.stream.writeln(">>> %s" % (self.getDescription(test)))
        self.stream.writeln(self.separator2)
        self.stream.writeln(self._exc_info_to_string(err, test).rstrip())
        self.stream.writeln(self.separator1)
        self.stream.writeln()

        ## Skip test runner traceback levels
        #while tb and self._is_relevant_tb_level(tb):
        #    tb = tb.tb_next

        # Really hacky way to jump up a couple of frames.
        # I'm sure it's not that difficult to do properly,
        # but I havn't figured out how.
        #p = pdb.Pdb()
        #p.reset()
        #p.setup(None, tb)
        #p.do_up(None)
        #p.do_up(None)
        #p.cmdloop()

        # It would be good if we could make sure we're in the correct frame here
        self.get_pdb().post_mortem(tb)


class PdbTestResult(ExceptionTestResultMixin, TextTestResult):
    pass


class PdbTestRunner(DjangoTestRunner):
    """
    Override the standard DjangoTestRunner to instead drop into pdb on test errors/failures.
    """
    def _makeResult(self):
        return PdbTestResult(self.stream, self.descriptions, self.verbosity)


class PdbTestSuiteRunner(DjangoTestSuiteRunner):
    """
    Override the standard DjangoTestSuiteRunner to instead drop into pdb on test errors/failures.
    """
    def run_suite(self, suite, **kwargs):
        return PdbTestRunner(verbosity=self.verbosity,
                             failfast=self.failfast).run(suite)


class IPdbTestResult(ExceptionTestResultMixin, TextTestResult):

    pdb_type = 'ipdb'


class IPdbTestRunner(DjangoTestRunner):
    """
    Override the standard DjangoTestRunner to instead drop into ipdb on test errors/failures.
    """
    def _makeResult(self):
        return IPdbTestResult(self.stream, self.descriptions, self.verbosity)


class IPdbTestSuiteRunner(DjangoTestSuiteRunner):
    """
    Override the standard DjangoTestSuiteRunner to instead drop into ipdb on test errors/failures.
    """

    def run_suite(self, suite, **kwargs):
        return IPdbTestRunner(verbosity=self.verbosity,
                              failfast=self.failfast).run(suite)



import inspect
import os
import pdb
import sys
from django_pdb.utils import get_ipdb, has_ipdb


class PdbMiddleware(object):
    """
    Middleware to break into pdb at the start of views.

    If `always_break` is set, due to `runserver --pdb` this will break
    into pdb at the start of every view.

    Otherwise it will break into pdb at the start of the view if
    a 'pdb' GET parameter is set on the request url.
    """

    always_break = False

    def get_type_pdb(self, request):
        type_pdb = None
        if self.always_break:
            type_pdb = self.always_break
        elif request.GET.get('pdb', None) is not None:
            type_pdb = 'pdb'
        elif request.GET.get('ipdb', None) is not None:
            type_pdb = 'ipdb'
        return type_pdb

    def process_view(self, request, view_func, view_args, view_kwargs):
        """
        If running with '--pdb', set a breakpoint at the start
        of each of each view before it gets called.
        """
        # Skip out unless using `runserver --pdb`,
        # or `pdb` is in the command line parameters
        type_pdb = self.get_type_pdb(request)
        if not type_pdb:
            return

        filename = inspect.getsourcefile(view_func)
        basename = os.path.basename(filename)
        dirname = os.path.basename(os.path.dirname(filename))
        lines, lineno = inspect.getsourcelines(view_func)
        temporary = True
        cond = None
        funcname = view_func.__name__

        print
        print '%s %s' % (request.method, request.get_full_path())
        print 'function "%s" in %s/%s:%d' % (funcname,
                                             dirname,
                                             basename,
                                             lineno)
        print 'args: %s' % (view_args,)
        print 'kwargs: %s' % (view_kwargs,)
        print

        if type_pdb == 'ipdb' and has_ipdb():
            p = get_ipdb()
        else:
            if not type_pdb == 'pdb':
                print 'You do not install ipdb or ipython module'
            p = pdb.Pdb()
        p.reset()
        p.set_break(filename, lineno + 1, temporary, cond, funcname)
        sys.settrace(p.trace_dispatch)



from django.core.management.commands.runserver import Command as RunServerCommand
from django_pdb.middleware import PdbMiddleware
from optparse import make_option


class Command(RunServerCommand):
    """
    Identical to Django's standard 'runserver' management command,
    except that it also adds support for a '--pdb' option.
    """
    option_list = RunServerCommand.option_list + (
        make_option('--pdb', action='store_true', dest='pdb', default=False,
            help='Drop into pdb shell on at the start of any view.'),
        make_option('--ipdb', action='store_true', dest='ipdb', default=False,
            help='Drop into ipdb shell on at the start of any view.'),
    )

    def handle(self, *args, **options):
        # Add pdb middleware, if --pdb is specified, or if we're in DEBUG mode
        from django.conf import settings

        pdb_option = options.pop('pdb')
        ipdb_option = options.pop('ipdb')

        if pdb_option or settings.DEBUG:
            settings.MIDDLEWARE_CLASSES += ('django_pdb.middleware.PdbMiddleware',)

        # If --pdb is specified then always break at the start of views.
        # Otherwise break only if a 'pdb' query parameter is set in the url.
        if pdb_option:
            PdbMiddleware.always_break = 'pdb'
        elif ipdb_option:
            PdbMiddleware.always_break = 'ipdb'

        super(Command, self).handle(*args, **options)



from django.core.management.commands.test import Command as TestCommand
from django_pdb.testrunners import PdbTestSuiteRunner
from django_pdb.testrunners import IPdbTestSuiteRunner
from optparse import make_option
import sys


class Command(TestCommand):
    """
    Identical to Django's standard 'test' management command,
    except that it also adds support for a '--pdb' option.
    """

    option_list = TestCommand.option_list + (
        make_option('--pdb', action='store_true', dest='pdb', default=False,
            help='Drop into pdb shell on test errors or failures.'),
        make_option('--ipdb', action='store_true', dest='ipdb', default=False,
            help='Drop into ipdb shell on test errors or failures.'),
    )

    pdb_testrunner = PdbTestSuiteRunner
    ipdb_testrunner = IPdbTestSuiteRunner

    def handle(self, *test_labels, **options):
        """
        If --pdb is set on the command line ignore the default test runner
        use the pdb test runner instead.
        """
        pdb = options.pop('pdb')
        ipdb = options.pop('ipdb')

        if pdb or ipdb:
            options['verbosity'] = int(options.get('verbosity', 1))

            if ipdb:
                test_runner = self.ipdb_testrunner(**options)
            else:
                test_runner = self.pdb_testrunner(**options)
            failures = test_runner.run_tests(test_labels)

            if failures:
                sys.exit(bool(failures))

        else:
            super(Command, self).handle(*test_labels, **options)



def has_ipdb():
    try:
        import ipdb
        import IPython
        return True
    except ImportError:
        return False


def get_ipdb():
    def_colors = get_def_colors()
    try:
        import ipdb
        from ipdb import __main__
        return ipdb.__main__.Pdb(def_colors)
    except ImportError:  # old versions of ipdb
        return ipdb.Pdb(def_colors)


def get_def_colors():
    # Inspirated in https://github.com/gotcha/ipdb/blob/master/ipdb/__main__.py
    def_colors = 'Linux'
    import IPython
    if IPython.__version__ > '0.10.2':
        from IPython.core.debugger import Pdb
        try:
            get_ipython
        except NameError:
            from IPython.frontend.terminal.embed import InteractiveShellEmbed
            ipshell = InteractiveShellEmbed()
            def_colors = ipshell.colors
        else:
            def_colors = get_ipython.im_self.colors
    else:
        from IPython.Debugger import Pdb
        from IPython.Shell import IPShell
        from IPython import ipapi
        ip = ipapi.get()
        if ip is None:
            IPShell(argv=[''])
            ip = ipapi.get()
        def_colors = ip.options.colors
    return def_colors



#!/usr/bin/env/python
# -*- coding: utf-8 -*-

from setuptools import setup
import os
import re

def abs_path(relative_path):
    """
    Given a path relative to this directory return an absolute path.
    """
    base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

def get_version(relative_path):
    """
    Return version given package's path.
    """
    data = open(os.path.join(abs_path(relative_path), '__init__.py')).read()
    return re.search(r"__version__ = '([^']+)'", data).group(1)

def get_long_description():
    """
    Return the contents of the README file.
    """
    try:
        return open(abs_path('README.rst')).read()
    except:
        pass # Required to install using pip (won't have README.rst then)


setup(
    name='django-pdb',
    version=get_version('django_pdb'),
    description='Gives you `manage.py runserver --pdb` and `manage.py test --pdb`',
    long_description=get_long_description(),
    author='Tom Christie',
    author_email='tom@tomchristie.com',
    url='https://github.com/tomchristie/django-pdb',
    packages=('django_pdb',
              'django_pdb.management',
              'django_pdb.management.commands'),
    license='Public Domain',
    classifiers = [
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Framework :: Django',
        'Operating System :: OS Independent',
    ],
)
